import { Serializer } from './Contracts';
/**
 * A decorator for a Storage implementation
 */
declare class Store {
    private driver;
    private serializer;
    /**
     * Create a Store instance.
     *
     * @param   {Storage}  driver        The storage driver
     * @param   {Object}   [serializer]  The serializer to use
     *
     * @return  {void}
     */
    constructor(driver: Storage, serializer?: Serializer);
    /**
     * Add a key/value pair to storage
     *
     * @param  {String}  key  The key to add
     * @param  {Mixed}   val  The value to add
     *
     * @throws {Error}  If the storage access is disallowed.
     *
     * @return {Store}
     */
    setItem(key: any, val: any): this;
    /**
     * Retrieve a value from storage by key
     *
     * @param  {String}  key  The key to retreive
     *
     * @return {Mixed}
     */
    getItem(key: any): any;
    /**
     * Determine whether the key exists in storage.
     *
     * @param   {String}   key  The key to check
     *
     * @return  {Boolean}
     */
    hasItem(key: any): boolean;
    /**
     * Remove an item from storage.
     *
     * @param   {String}  key  The key to remove
     *
     * @return  {Boolean}
     */
    removeItem(key: any): boolean;
    /**
     * Remove all items from storage.
     *
     * @return  {Store}
     */
    clear(): this;
    /**
     * Iterate through the items in storage and execute the callback.
     *
     * @param   {Function}  callback            The callback function
     * @param   {Object}    [thisContext=this]  The context of this keyword
     *
     * @return  {void}
     */
    forEach(callback: Function, thisContext?: this): void;
    /**
     * Determine whether the storage driver is supported.
     *
     * @return  {Boolean}
     */
    isSupported(): boolean;
    /**
     * Get the number of items in storage.
     *
     * @return  {Integer}
     */
    readonly length: number;
}
export default Store;
