'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */



var __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
};

var commonjsGlobal = typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};



function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var chitu = createCommonjsModule(function (module, exports) {
(function (global, factory) {
    factory(exports);
}(commonjsGlobal, (function (exports) { 'use strict';

/**
 * Module for checking variable types.
 *
 * @module type
 */
var type = {};
var types = 'Array Object String Date RegExp Function Boolean Number Error Null Undefined Symbol'.split(' ');
/**
 * Check the type of a value.
 *
 * @param  {String}  type  The primitive type as a capitalized string.
 * @param  {Mixed}   val   The value to check.
 *
 * @return {Boolean}
 */
type.is = function (type, val) {
  return Object.prototype.toString.call(val).slice(8, -1) === type;
};

// Populate helper methods.
types.forEach(function (t) {
  return type['is' + t] = function (v) {
    return type.is(t, v);
  };
});

/**
 * Return the value or execute as a function
 *
 * @module  value
 *
 * @param   {Mixed}  val     A value or a function that returns a value
 * @param   {Mixed}  [args]  One or more arguments
 *
 * @return  {Mixed}
 */
function value(val) {
  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  return type.isFunction(val) ? val.apply(this, args) : val;
}

/**
 * Round a number to a given number of decimal places.
 *
 * @param   {Number}  val         The number to round
 * @param   {Number}  [places=2]  The number of places to round to
 *
 * @return  {Number}
 */
function round(val) {
  var places = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 2;

  return Number(val.toFixed(places));
}

function ordinal(num) {
    var suffix = ['th', 'st', 'nd', 'rd'];
    var v = num % 100;

    return num + (suffix[(v - 20) % 10] || suffix[v] || suffix[0]);
}

exports.type = type;
exports.value = value;
exports.round = round;
exports.ordinal = ordinal;

Object.defineProperty(exports, '__esModule', { value: true });

})));

});

var chitu$1 = unwrapExports(chitu);

/**
 * Serialize and Unserialize data
 */
var JSONSerializer = {
    /**
     * Serialize the provided data.
     *
     * @param   {Mixed}  data  The data to serialize
     *
     * @return  {String}
     */
    serialize: function (data) {
        try {
            return JSON.stringify(data);
        }
        catch (e) {
            return data;
        }
    },
    /**
     * Unserialize the provided data.
     *
     * @param   {String}  payload  The payload to unserialize
     *
     * @return  {Mixed}
     */
    unserialize: function (payload) {
        try {
            return JSON.parse(payload);
        }
        catch (e) {
            return payload;
        }
    }
};

/**
 * A decorator for a Storage implementation
 */
var Store = /** @class */ (function () {
    /**
     * Create a Store instance.
     *
     * @param   {Storage}  driver        The storage driver
     * @param   {Object}   [serializer]  The serializer to use
     *
     * @return  {void}
     */
    function Store(driver, serializer) {
        this.driver = driver;
        this.serializer = serializer || JSONSerializer;
    }
    /**
     * Add a key/value pair to storage
     *
     * @param  {String}  key  The key to add
     * @param  {Mixed}   val  The value to add
     *
     * @throws {Error}  If the storage access is disallowed.
     *
     * @return {Store}
     */
    Store.prototype.setItem = function (key, val) {
        try {
            this.driver.setItem(key, this.serializer.serialize(val));
        }
        catch (e) {
            if ([
                'QUOTA_EXCEEDED_ERR',
                'NS_ERROR_DOM_QUOTA_REACHED',
                'QuotaExceededError'
            ].indexOf(e.name) !== -1) {
                throw new Error('The Storage quota has been exceeded');
            }
            else {
                throw new Error("Could not add item with key \"" + key + "\"");
            }
        }
        return this;
    };
    /**
     * Retrieve a value from storage by key
     *
     * @param  {String}  key  The key to retreive
     *
     * @return {Mixed}
     */
    Store.prototype.getItem = function (key) {
        return this.serializer.unserialize(this.driver.getItem(key));
    };
    /**
     * Determine whether the key exists in storage.
     *
     * @param   {String}   key  The key to check
     *
     * @return  {Boolean}
     */
    Store.prototype.hasItem = function (key) {
        return this.driver.hasOwnProperty(key) || !!this.getItem(key);
    };
    /**
     * Remove an item from storage.
     *
     * @param   {String}  key  The key to remove
     *
     * @return  {Boolean}
     */
    Store.prototype.removeItem = function (key) {
        if (!this.hasItem(key))
            return false;
        this.driver.removeItem(key);
        return true;
    };
    /**
     * Remove all items from storage.
     *
     * @return  {Store}
     */
    Store.prototype.clear = function () {
        this.driver.clear();
        return this;
    };
    /**
     * Iterate through the items in storage and execute the callback.
     *
     * @param   {Function}  callback            The callback function
     * @param   {Object}    [thisContext=this]  The context of this keyword
     *
     * @return  {void}
     */
    Store.prototype.forEach = function (callback, thisContext) {
        if (thisContext === void 0) { thisContext = this; }
        for (var item in this.driver) {
            callback.call(thisContext, this.driver[item], item);
        }
    };
    /**
     * Determine whether the storage driver is supported.
     *
     * @return  {Boolean}
     */
    Store.prototype.isSupported = function () {
        var l = 'l';
        try {
            this.driver.setItem(l, l);
            this.driver.removeItem(l);
            return true;
        }
        catch (e) {
            return false;
        }
    };
    Object.defineProperty(Store.prototype, "length", {
        /**
         * Get the number of items in storage.
         *
         * @return  {Integer}
         */
        get: function () {
            return this.driver.length;
        },
        enumerable: true,
        configurable: true
    });
    return Store;
}());

var type = chitu$1.type;
var value = chitu$1.value;
/**
 * A Fluent Storage API.
 */
var Lkr = /** @class */ (function () {
    /**
     * Create a Lkr instance.
     *
     * @param   {Object}  options               The configuration options
     * @param   {Object}  options.drivers       The storage drivers
     * @param   {String}  options.driver        The default storage driver
     * @param   {String}  options.namespace     The namespace
     * @param   {String}  options.separator     The string that separates the namespace and key
     * @param   {Object}  [options.serializer]  The serializer to use
     *
     * @return  {void}
     */
    function Lkr(options) {
        /**
         * The configuration options.
         *
         * @type  {LkrOptions}
         */
        this.opts = options;
        /**
         * The Store instance.
         *
         * @private
         *
         * @type {Store}
         */
        this.store = (function (driver, self) {
            if (!Object.keys(options.drivers).length ||
                !options.drivers.hasOwnProperty(driver)) {
                throw new Error("Driver \"" + driver + "\" not available.");
            }
            var store = new Store(options.drivers[driver], options.serializer);
            if (!store.isSupported()) {
                throw new Error("Driver \"" + driver + "\" not supported.");
            }
            return store;
        })(options.driver, this);
    }
    /**
     * Get the fully qualified key.
     *
     * @private
     *
     * @param  {String}  key  The key
     *
     * @return {String}
     */
    Lkr.prototype.getKey = function (key) {
        var _a = this.opts, namespace = _a.namespace, separator = _a.separator;
        return namespace ? "" + namespace + separator + key : key;
    };
    /**
     * Add a new item to storage even if it already exists.
     *
     * @param  {String|Function|Object}  key    The key to add
     * @param  {Mixed}                   val    The value to add
     * @param  {Mixed}                   [def]  The default to pass to function if doesn't already exist
     *
     * @throws {Error}  If a key or value is not provided
     *
     * @return {Lkr}
     */
    Lkr.prototype.put = function (key, val, def) {
        if (type.isUndefined(key))
            throw new Error('You must specify a key.');
        key = value(key);
        if (type.isObject(key)) {
            for (var item in key) {
                var v = value(key[item]);
                this.store.setItem(this.getKey(item), type.isUndefined(v) ? def : v);
            }
        }
        else {
            if (type.isUndefined(val))
                throw new Error('You must specify a value.');
            this.store.setItem(this.getKey(key), value(val, this.get(key, def)));
        }
        return this;
    };
    /**
     * Add an item to storage if it doesn't already exist.
     *
     * @param  {String|Function|Object}  key      The key to add
     * @param  {Mixed}                   value    The value to add
     * @param  {Mixed}                   [def]    The default to pass to function if doesn't already exist
     *
     * @return {Boolean}  Whether the item was added or not
     */
    Lkr.prototype.add = function (key, value, def) {
        if (!this.has(key)) {
            this.put(key, value, def);
            return true;
        }
        return false;
    };
    /**
     * Retrieve the specified item from storage.
     *
     * @param  {String|Array|Function}  key    The key to get
     * @param  {Mixed}                  [def]  The default value if it does not exist
     *
     * @return {Mixed}
     */
    Lkr.prototype.get = function (key, def) {
        key = value(key);
        if (type.isArray(key)) {
            var items = {};
            for (var _i = 0, key_1 = key; _i < key_1.length; _i++) {
                var k = key_1[_i];
                items[k] = this.get(k, def);
            }
            return items;
        }
        if (!this.has(key))
            return arguments.length === 2 ? def : void 0;
        return this.store.getItem(this.getKey(key));
    };
    /**
     * Determine whether the item exists in storage.
     *
     * @param  {String|Function}  key  The key to remove
     *
     * @return {Boolean}
     */
    Lkr.prototype.has = function (key) {
        return this.store.hasItem(value(this.getKey(key)));
    };
    /**
     * Remove specified item(s) from storage.
     *
     * @param  {String|Array|Function}  key  The key or array of keys to remove
     *
     * @return {Lkr}
     */
    Lkr.prototype.forget = function (key) {
        key = value(key);
        if (type.isArray(key)) {
            key.map(this.forget, this);
        }
        else {
            this.store.removeItem(this.getKey(key));
        }
        return this;
    };
    /**
     * Retrieve the specified item from storage and then remove it
     *
     * @param  {String|Array}  key    The key to pull from storage
     * @param  {Mixed}         [def]  The default value if it does not exist
     *
     * @return {Mixed}
     */
    Lkr.prototype.pull = function (key, def) {
        var value = this.get(key, def);
        this.forget(key);
        return value;
    };
    /**
     * Get all the items in storage within the currently set namespace/driver.
     *
     * @return  {Object}
     */
    Lkr.prototype.all = function () {
        var _this = this;
        var items = {};
        this.store.forEach(function (val, key) {
            if (_this.opts.namespace) {
                var prefix = _this.opts.namespace + _this.opts.separator;
                if (key.indexOf(prefix) === 0)
                    key = key.substring(prefix.length);
            }
            if (_this.has(key))
                items[key] = _this.get(key);
        }, this);
        return items;
    };
    /**
     * Iterate through the items within the current namespace/driver and execute the callback.
     *
     * @param   {Function}  callback            The callback function
     * @param   {Object}    [thisContext=this]  The context of this keyword
     *
     * @return  {void}
     */
    Lkr.prototype.each = function (callback, thisContext) {
        if (thisContext === void 0) { thisContext = this; }
        var items = this.all();
        for (var item in items) {
            callback.call(thisContext, items[item], item);
        }
    };
    /**
     * Get the storage keys as an array.
     *
     * @return {Array}
     */
    Lkr.prototype.keys = function () {
        return Object.keys(this.all());
    };
    /**
     * Remove all items set within the current namespace/driver.
     *
     * @return {Lkr}
     */
    Lkr.prototype.clean = function () {
        return this.forget(this.keys());
    };
    /**
     * Empty the current storage driver completely. Careful now.
     *
     * @return {Lkr}
     */
    Lkr.prototype.empty = function () {
        this.store.clear();
        return this;
    };
    /**
     * Get the total number of items within the current namespace/driver.
     *
     * @return {Integer}
     */
    Lkr.prototype.count = function () {
        return this.keys().length;
    };
    /**
     * Set the driver by key.
     *
     * @param   {String}  driver  The driver key
     *
     * @return  {Lkr}
     */
    Lkr.prototype.driver = function (driver) {
        return Lkr.make(__assign({}, this.opts, { driver: driver }));
    };
    /**
     * Set the namespace.
     *
     * @param   {String}  namespace  The namespace
     *
     * @return  {Lkr}
     */
    Lkr.prototype.namespace = function (namespace) {
        return Lkr.make(__assign({}, this.opts, { namespace: namespace }));
    };
    /**
     * Create a new instance of Lkr.
     *
     * @param   {Object}  options  The configuration options
     *
     * @return  {Lkr}
     */
    Lkr.make = function (options) {
        return new Lkr(options);
    };
    return Lkr;
}());

var Locker = new Lkr({
    drivers: {
        local: window.localStorage,
        session: window.sessionStorage
    },
    driver: 'local',
    namespace: 'lkr',
    separator: '.'
});

exports.Lkr = Lkr;
exports.Locker = Locker;
//# sourceMappingURL=lkr.js.map
